/**
 * Representação de uma entidade o ECS
 */
export declare abstract class Entity {
    static sequence(): number;
    /**
     * Lista de interessados sobre a atualiação dos componentes
     */
    private subscriptions;
    /**
     *
     */
    id: number;
    /**
     * Informs that subscriptions have been triggered.
     */
    private queued;
    /**
     * Informa se a entidade está ativa
     */
    active: boolean;
    /**
     * Componentes por tipo
     *
     * ATENÇÃO! Apesar de a lista de componentes ser publica, não fazer modificação na mesma, utilizar os metodos add e
     * remove para tal
     */
    components: {
        [key: number]: Component<any>[];
    };
    constructor();
    /**
     *
     */
    private dispatch;
    /**
     * Permite aos interessados receber informação quando a lista de componentes dessa entidade receber atualização
     *
     * @param handler
     */
    subscribe(handler: () => void): () => Entity;
    /**
     * Adiciona um componente à entidade
     *
     * @param component
     */
    add(component: Component<any>): void;
    /**
     * Remove a referencia de um componente dessa entidade
     *
     * @param component
     */
    remove(component: Component<any>): void;
}
/**
 * Force typing
 */
export declare type ComponentClassType<P> = (new (data: P) => Component<P>) & {
    type: number;
    allFrom(entity: Entity): Component<P>[];
    oneFrom(entity: Entity): Component<P>;
};
/**
 * Informações sobre um componente
 */
export declare class Component<T> {
    /**
     * Registra uma nova classe de componente
     */
    static register<P>(): ComponentClassType<P>;
    /**
     * Obtem o id do tipo do componente
     */
    type: number;
    /**
     * Obtem o id do tipo do componente
     */
    data: T;
    constructor(type: number, data: T);
}
/**
 * Represents the logic that transforms component data of an entity from its current state to its next state. A system
 * runs on entities that have a specific set of component types.
 */
export declare abstract class System {
    /**
     * Identificador único de uma instancia deste sistema
     */
    id: number;
    /**
     * IDs of the types of components this system expects the entity to have before it can act on. If you want to
     * create a system that acts on all entities, enter [-1]
     */
    private components;
    /**
     * The maximum times per second this system should be updated.
     */
    frequence: number;
    /**
     * Invoked in updates, limited to the value set in the "frequency" attribute
     *
     * @param entity Identificador da entidade
     * @param components De uma entidade
     */
    update?(time: number, delta: number, entity: Entity): void;
    /**
     * Invoked when:
     * a) An entity with the characteristics (components) expected by this system is added in the world;
     * b) This system is added in the world and this world has one or more entities with the characteristics expected by
     * this system;
     * c) An existing entity in the same world receives a new component at runtime and all of its new components match
     * the standard expected by this system.
     *
     * @param entity
     * @param components
     */
    enter?(entity: Entity): void;
    /**
     * Invoked when:
     * a) An entity with the characteristics (components) expected by this system is removed from the world;
     * b) This system is removed from the world and this world has one or more entities with the characteristics
     * expected by this system;
     * c) An existing entity in the same world loses a component at runtime and its new component set no longer matches
     * the standard expected by this system
     *
     * @param entity
     * @param components
     */
    exit?(entity: Entity): void;
    /**
     * @param components IDs of the types of components this system expects the entity to have before it can act on.
     * If you want to create a system that acts on all entities, enter [-1]
     * @param frequence The maximum times per second this system should be updated. Defaults 0
     */
    constructor(components: number[], frequence?: number);
    getComponents(): number[];
}
/**
 * A propria definição do ECS. Também chamada de Admin ou Manager em outras implementações
 */
export default class ECS {
    static System: typeof System;
    static Entity: typeof Entity;
    static Component: typeof Component;
    /**
     * Todos os Systems existentes nesse mundo
     */
    private systems;
    /**
     * Todas as entidades existentes neste mundo
     */
    private entities;
    /**
     * Indexa os sistemas que devem ser executados para cada entidade
     */
    private entitySystems;
    /**
     * Registra o último instante que um sistema foi executado neste mundo para uma entidade
     */
    private entitySystemLastUpdate;
    /**
     * Guarda as subscrições realizadas para entidades
     */
    private entitySubscription;
    constructor(systems?: System[]);
    /**
     * Obtém uma entidade por id
     *
     * @param id
     */
    getEntity(id: number): Entity | undefined;
    /**
     * Adiciona uma entidade nesse mundo
     *
     * @param entity
     */
    addEntity(entity: Entity): void;
    /**
     * Remove uma entidade desse mundo
     *
     * @param idOrInstance
     */
    removeEntity(idOrInstance: number | Entity): void;
    /**
     * Adiciona o sistema nesse mundo
     *
     * @param system
     */
    addSystem(system: System): void;
    /**
     * Remove o sistema desse mundo
     *
     * @param system
     */
    removeSystem(system: System): void;
    /**
     * Realiza a atualização dos sistemas desse mundo
     */
    update(): void;
    private indexEntitySystem;
    /**
     * Faz a indexação de uma entidade
     *
     * @param entity
     */
    private indexEntity;
}
